static const char *RcsId = "$Header: /home/cvsadm/cvsroot/fermi/servers/a2505bs/src/A2505BS.cpp,v 1.4 2019-11-13 12:53:43 claudio Exp $";
//+=============================================================================
//
// file :         A2505BS.cpp
//
// description :  C++ source for the A2505BS and its commands. 
//                The class is derived from Device. It represents the
//                CORBA servant object which will be accessed from the
//                network. All commands which can be executed on the
//                A2505BS are implemented in this file.
//
// project :      TANGO Device Server
//
// $Author: claudio $
//
// $Revision: 1.4 $
//
// $Log: A2505BS.cpp,v $
// Revision 1.4  2019-11-13 12:53:43  claudio
// TANGO_BASE_CLASS, tango-9.3.3
//
// Revision 1.3  2013-09-03 07:57:12  giulio
// Moved to Tango8 and Device4Impl
//
// Revision 1.2  2010-02-16 10:13:35  graziano
// removed debug
//
// Revision 1.2  2008/01/30 10:31:07  enrico
// *** empty log message ***
//
// Revision 1.1  2007/12/12 11:44:38  enrico
// Release version. ME
//
//
// copyleft :   Sincrotrone Trieste S.C.p.A.
//              Strada Statale 14 - km 163,5 in AREA Science Park
//              34012 Basovizza, Trieste
//              ITALY
//
//-=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================



//===================================================================
//
//	The following table gives the correspondance
//	between commands and method's name.
//
//  Command's name   |  Method's name
//	----------------------------------------
//  State            |  dev_state()
//  Status           |  dev_status()
//  GetID            |  get_id()
//  On               |  on()
//  Off              |  off()
//  GetCurrent       |  get_current()
//  GetVoltage       |  get_voltage()
//  GetDACValue      |  get_dacvalue()
//  GetReference     |  get_reference()
//  GetTemperature   |  get_temperature()
//  GetPositiveRail  |  get_positive_rail()
//  GetStatus        |  get_status()
//  Reset            |  reset()
//  ModGlobStat      |  mod_glob_stat()
//  DisableSwitcher  |  disable_switcher()
//  EnableSwitcher   |  enable_switcher()
//  ClearStatus      |  clear_status()
//  Standby          |  standby()
//  StartCycling     |  start_cycling()
//  Abort            |  abort()
//
//===================================================================


#include <tango.h>
#include <A2505BS.h>
#include <A2505BSClass.h>

//#define DEBUG

#ifndef DEBUG
#define dprintf //
#else
#define dprintf printf
#endif

#define MODEL_ID 2505
const char LF_STRING[]={0x10, 0};

namespace A2505BS_ns
{

//+----------------------------------------------------------------------------
//
// method : 		A2505BS::A2505BS(string &s)
// 
// description : 	constructor for simulated A2505BS
//
// in : - cl : Pointer to the DeviceClass object
//      - s : Device name 
//
//-----------------------------------------------------------------------------
A2505BS::A2505BS(Tango::DeviceClass *cl,string &s)
:TANGO_BASE_CLASS(cl,s.c_str())
{
	init_device();
}

A2505BS::A2505BS(Tango::DeviceClass *cl,const char *s)
:TANGO_BASE_CLASS(cl,s)
{
	init_device();
}

A2505BS::A2505BS(Tango::DeviceClass *cl,const char *s,const char *d)
:TANGO_BASE_CLASS(cl,s,d)
{
	init_device();
}
//+----------------------------------------------------------------------------
//
// method : 		A2505BS::delete_device()
// 
// description : 	will be called at device destruction or at init command.
//
//-----------------------------------------------------------------------------
void A2505BS::delete_device()
{
	//	Delete device's allocated object
	if (attr_Temperature_read)
	{
		delete attr_Temperature_read;
		attr_Temperature_read = 0;
	}
	if (attr_Current_read)
	{
		delete attr_Current_read;
		attr_Current_read = 0;
	}
	if (attr_Voltage_read)
	{
		delete attr_Voltage_read;
		attr_Voltage_read = 0;
	}
	if (attr_CurrentSetPoint_read)
	{
		delete attr_CurrentSetPoint_read;
		attr_CurrentSetPoint_read = 0;
	}
	if (attr_ModelID_read)
	{
		delete attr_ModelID_read;
		attr_ModelID_read = 0;
	}	
	if (sockfd) 
	{
		close(sockfd);
		sockfd = 0;
	}
}

//+----------------------------------------------------------------------------
//
// method : 		A2505BS::init_device()
// 
// description : 	will be called at device initialization.
//
//-----------------------------------------------------------------------------
void A2505BS::init_device()
{
	INFO_STREAM << "A2505BS::A2505BS() create device " << device_name << endl;

	// Initialise variables to default values
	//--------------------------------------------
	get_device_property();
	char tmp[STRING_SIZE];
	try {
		init_attr_double(attr_Voltage_read);
		init_attr_double(attr_Current_read);
		init_attr_double(attr_Temperature_read);
		init_attr_double(attr_CurrentSetPoint_read);
		init_attr_long(attr_ModelID_read);
		
		Tango::MultiAttribute *attr_list = get_device_attr();
		attr_set_current = 0;
		attr_set_current = &(attr_list->get_w_attr_by_name("Current"));
		
		attr_PSStatus_read = &statusArray[0];
		memset (response, 0, BUF_LEN);
		dprintf("openSocket\n");
		sprintf(tmp, "maxCurr: %2.3f A", maxCurr);
		fileLogMsg(tmp);
		sprintf(tmp, "minCurr: %2.3f A", minCurr);
		fileLogMsg(tmp);
		sprintf(tmp, "StandbyCurr: %2.3f A", standbyCurrent);
		fileLogMsg(tmp);
		sprintf(tmp, "CycMaxCurr: %2.3f A", cycMaxCurr);
		fileLogMsg(tmp);
		sprintf(tmp, "CycMinCurr: %2.3f A", cycMinCurr);
		fileLogMsg(tmp);
		sprintf(tmp, "CycEndCurr: %2.3f A", cycEndCurr);
		fileLogMsg(tmp);
		sprintf(tmp, "CycleWait: %d ms", cycleWait);
		fileLogMsg(tmp);
		openSocket();
	}
	catch(Tango::DevFailed &e) {
		set_state(Tango::FAULT);
		set_status("Device FAULT");
		TangoSys_MemStream out_stream;
		out_stream << "init_device() failed." << ends;
		cout<<out_stream.str()<<endl;
		return;
	}
	
}

//+----------------------------------------------------------------------------
//
// method : 		A2505BS::get_device_property()
// 
// description : 	Read the device properties from database.
//
//-----------------------------------------------------------------------------
void A2505BS::get_device_property()
{
	//	Initialize your default values here (if not done with  POGO).
	//------------------------------------------------------------------

	//	Read device properties from database.(Automatic code generation)
	//------------------------------------------------------------------
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("IpAddress"));
	dev_prop.push_back(Tango::DbDatum("TcpPort"));
	dev_prop.push_back(Tango::DbDatum("StandbyCurrent"));
	dev_prop.push_back(Tango::DbDatum("MaxCurr"));
	dev_prop.push_back(Tango::DbDatum("MinCurr"));
	dev_prop.push_back(Tango::DbDatum("CycMaxCurr"));
	dev_prop.push_back(Tango::DbDatum("CycMinCurr"));
	dev_prop.push_back(Tango::DbDatum("CycEndCurr"));
	dev_prop.push_back(Tango::DbDatum("CycleWait"));

	//	Call database and extract values
	//--------------------------------------------
	if (Tango::Util::instance()->_UseDb==true)
		get_db_device()->get_property(dev_prop);
	Tango::DbDatum	def_prop, cl_prop;
	A2505BSClass	*ds_class =
		(static_cast<A2505BSClass *>(get_device_class()));
	int	i = -1;

	//	Try to initialize IpAddress from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  ipAddress;
	//	Try to initialize IpAddress from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  ipAddress;
	//	And try to extract IpAddress value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  ipAddress;

	//	Try to initialize TcpPort from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  tcpPort;
	//	Try to initialize TcpPort from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  tcpPort;
	//	And try to extract TcpPort value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  tcpPort;

	//	Try to initialize StandbyCurrent from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  standbyCurrent;
	//	Try to initialize StandbyCurrent from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  standbyCurrent;
	//	And try to extract StandbyCurrent value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  standbyCurrent;

	//	Try to initialize MaxCurr from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  maxCurr;
	//	Try to initialize MaxCurr from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  maxCurr;
	//	And try to extract MaxCurr value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  maxCurr;

	//	Try to initialize MinCurr from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  minCurr;
	//	Try to initialize MinCurr from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  minCurr;
	//	And try to extract MinCurr value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  minCurr;

	//	Try to initialize CycMaxCurr from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  cycMaxCurr;
	//	Try to initialize CycMaxCurr from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  cycMaxCurr;
	//	And try to extract CycMaxCurr value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  cycMaxCurr;

	//	Try to initialize CycMinCurr from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  cycMinCurr;
	//	Try to initialize CycMinCurr from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  cycMinCurr;
	//	And try to extract CycMinCurr value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  cycMinCurr;

	//	Try to initialize CycEndCurr from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  cycEndCurr;
	//	Try to initialize CycEndCurr from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  cycEndCurr;
	//	And try to extract CycEndCurr value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  cycEndCurr;

	//	Try to initialize CycleWait from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  cycleWait;
	//	Try to initialize CycleWait from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  cycleWait;
	//	And try to extract CycleWait value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  cycleWait;



	//	End of Automatic code generation
	//------------------------------------------------------------------

}

void A2505BS::init_attr_double(Tango::DevDouble*& attr)
{
	attr = 0;
	attr = new Tango::DevDouble;
	if (attr == 0) {
		Tango::Except::throw_exception (
				(const char *)"OUT OF MEMORY",
		(const char *)"out of memory error",
		(const char *)"A2505BS::init_attr_double()");
	}
}

void A2505BS::init_attr_long(Tango::DevLong*& attr)
{
	attr = 0;
	attr = new Tango::DevLong;
	if (attr == 0) {
		Tango::Except::throw_exception (
		(const char *)"OUT OF MEMORY",
		(const char *)"out of memory error",
		(const char *)"A2505BS::init_attr_long()");
	}
}

void A2505BS::openSocket()
{
	tv.tv_sec = 1;  /* 1 Sec Timeout */
	tv.tv_usec = 0;
	char tmp[STRING_SIZE];

	if ((sockfd = socket (AF_INET, SOCK_STREAM, 0)) < 0)
	{
		fprintf (stderr, "Error creating listening socket.\n");
		Tango::Except::throw_exception (
		(const char *)"SOCKET ERROR",
		(const char *)"socket() error",
		(const char *)"A2505BS::openSocket()");
	}

	if (setsockopt(sockfd, SOL_SOCKET, SO_RCVTIMEO, (struct timeval *)&tv, sizeof(struct timeval)) != 0 )
		DEBUG_STREAM << "Error setting RX timeout " << endl; 
	dprintf ("socket created (id %d)\n", sockfd);
	memset ((char *) &serv_addr, 0, sizeof (serv_addr));
	serv_addr.sin_family = AF_INET;
	serv_addr.sin_port = htons (tcpPort);
	serv_addr.sin_addr.s_addr = inet_addr (ipAddress.c_str());
	if (connect (sockfd, (struct sockaddr *) &serv_addr, sizeof (serv_addr)) < 0)
	{
		//fprintf(stderr, "************************\n");
		fprintf(stderr, "*   ERROR connecting   *\n");
		fprintf(stderr, "*  address: %s  *\n*  socket port: %d  *\n", ipAddress.c_str(), tcpPort);
		//fprintf(stderr, "************************\n");
        	Tango::Except::throw_exception (
        	(const char *)"SOCKET ERROR",
        	(const char *)"connect() error",
        	(const char *)"A2505BS::openSocket()");
	}
	else
	{
		sprintf(tmp,"Connected on %s port %d\n", ipAddress.c_str(), tcpPort);
		fileLogMsg(tmp);
	}
}

  void A2505BS::sendCmd (Tango::DevString cmd)
  {
	int nByte = 0;
	string command (cmd);
	command += CR;
	command += LF;

	if ((nByte = write (sockfd, command.c_str (), command.length ())) < 0)
	{
		dprintf ("ERROR writing to socket!\n");
		// fare blocco try?
		close(sockfd);
		openSocket();
		reset();
		off();
		//end blocco try
		Tango::Except::throw_exception (
		(const char *)"SOCKET ERROR",
		(const char *)"write() error",
		(const char *)"A2505BS::sendCmd()");
	}
	else 
		dprintf("write: |%s| - %d byte\n", command.c_str(), command.length() );
}


void A2505BS::getResponse()
{
	int nByte = 0;
	bool end = false;
	memset (response, 0, BUF_LEN);
	while (!end)
	{
		nByte = 0;
		memset (recvBuf, 0, BUF_LEN);
		nByte = recv (sockfd, recvBuf, BUF_LEN, 0);
		/*if (nByte == 0)
		{
			dprintf("\nSocket EOF\n");
			close(sockfd);
			sockfd = 0;
		}
		*/
		if (nByte < 0)
		{
			end = true;
			if (errno != EWOULDBLOCK)
			{
				set_state (Tango::FAULT);
				set_status("Device FAULT, error receiving from socket.");
				TangoSys_MemStream os;
				dprintf("ERROR READING!\n");
				os <<"Failed to get TCP response from node,"
				   <<"unexpected close of connection at remote end"
				   << ends;
				Tango::Except::throw_exception(
					(const char *) "error while reading response",
					os.str(),
					(const char *) "A2505BS::getResponse()");
				dprintf ("CLOSING SOCKET...\n");
				close (sockfd);
				sockfd = 0;
			}
			else
			{
				printf("\nsocket() failed on %d, errno %d\n", nByte, errno);
				set_state(Tango::FAULT);
				set_status("COMMUNICATION ERROR (check cable)");
				ERROR_STREAM<<"Communication error - check cable connection"<<endl;
				TangoSys_MemStream os;
				os <<"check cable connection" <<ends;
				Tango::Except::throw_exception(
				(const char *) "error while reading response",
				os.str(),
				(const char *) "A2505BS::getResponse()");
			}
		}
		else
		{
			//CR: 0xD;
			dprintf("nByte: %d\n", nByte);
			
			if ( (recvBuf[nByte-1] == LF) ) //ultimo=LF: messaggio terminato
			{
				end = true;
				if (nByte>1)
					strncat (response, recvBuf, (nByte-2));
			}
			else if ( (recvBuf[nByte-1] == CR) ) //ultimo (unico) carattere nel prossimo pacchetto
			{
				strncat (response, recvBuf, (nByte-1));
			}
			else
				strncat (response, recvBuf, nByte);

			for (int i=0; i<nByte; i++)
				dprintf("recvBuf[%d]: %x - %c\n", i, recvBuf[i], recvBuf[i]);
			dprintf("strlen(response): %d\n", strlen(response));
			for (int i=0; i<strlen(response); i++)
			dprintf("response[%d]: %x - %c\n", i, response[i], response[i]);
		}
	}
}

void A2505BS::analyzeMRGResponse(Tango::DevDouble & a)
{
	char *result = NULL;
	result = strtok (response, ":");
	result = strtok (NULL, ":");
	result = strtok (NULL, ":");
	if (result != NULL)
		a = strtod (result, NULL);
	dprintf("analyzeMRGResponse double: %f\n", a);
}

void A2505BS::analyzeResponse(Tango::DevDouble & a)
{
	char *result = NULL;
	result = strtok (response, ":");
	result = strtok (NULL, ":");
	if (result != NULL)
		  a = strtod (result, NULL);
	dprintf("analyzeResponse double: %f\n", a);
	/*
	while( result != NULL ) {
		dprintf( "result is \"%s\"\n", result );
		result = strtok( NULL, ":" );
		if ( result != NULL )
		a = strtof(result,NULL);
		cout<<"a: "<<a<<endl;
	}
	*/
}

void A2505BS::analyzeResponse(Tango::DevLong & a)
{
	char *result = NULL;
	result = strtok (response, ":");
	result = strtok (NULL, ":");
	if (result != NULL)
		a = strtol (result, NULL, 10);
}

void A2505BS::analyzeResponse()
{
	char tmp[STRING_SIZE];
	memset(tmp, 0, STRING_SIZE);
	if (strcmp (response, "#AK") && strcmp (response, "#MREADY"))
	{
		strcat(tmp,"analyzeResponse error: ");
		strcat(tmp, response);
		strcat(tmp,"!");
		dprintf("%s\n", tmp);
		Tango::Except::throw_exception (
		(const char *)"NOT ACKNOWLEDGED",
		tmp,
		(const char *)"A2505BS::analyzeResponse(void)");
	}
	else strcpy(response, "NULL");
}

//+----------------------------------------------------------------------------
//
// method : 		A2505BS::always_executed_hook()
// 
// description : 	method always executed before any command is executed
//
//-----------------------------------------------------------------------------
void A2505BS::always_executed_hook()
{
	get_status();
}
//+----------------------------------------------------------------------------
//
// method : 		A2505BS::read_attr_hardware
// 
// description : 	Hardware acquisition for attributes.
//
//-----------------------------------------------------------------------------
void A2505BS::read_attr_hardware(vector<long> &attr_list)
{
	DEBUG_STREAM << "A2505BS::read_attr_hardware(vector<long> &attr_list) entering... "<< endl;
	//	Add your own code here
	/*
	int state = get_state();
	if ( (state == Tango::OFF) //|| (state == Tango::STANDBY)
		     || (state == Tango::FAULT) ) {
		Tango::Except::throw_exception(
			(const char *) "A2505BSDeviceException",
			(const char *) "Attempt to read A2505BS device when off/fault.",
			(const char *) "Device must be ON or STANDBY before reading.",Tango::ERR);
}*/
}
//+----------------------------------------------------------------------------
//
// method : 		A2505BS::read_ModelID
// 
// description : 	Extract real attribute values for ModelID acquisition result.
//
//-----------------------------------------------------------------------------
void A2505BS::read_ModelID(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2505BS::read_ModelID(Tango::Attribute &attr) entering... "<< endl;
	*attr_ModelID_read = MODEL_ID;
	attr.set_value(attr_ModelID_read,1,0,false);
}


//+----------------------------------------------------------------------------
//
// method : 		A2505BS::read_CurrentSetPoint
// 
// description : 	Extract real attribute values for CurrentSetPoint acquisition result.
//
//-----------------------------------------------------------------------------
void A2505BS::read_CurrentSetPoint(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2505BS::read_CurrentSetPoint(Tango::Attribute &attr) entering... "<< endl;
	Tango::DevDouble argout = 0;
	try {
		sendCmd("MRG:36");
		getResponse();
		analyzeMRGResponse(argout);
	}
	catch(Tango::DevFailed &e) {
		Tango::Except::re_throw_exception(e,
				(const char *)"Command failed",
				(const char *)"Error from A2505BS",
				(const char *)"A2505BS::read_CurrentSetPoint()", Tango::ERR);
	}
	*attr_CurrentSetPoint_read = (double)argout/1000;
	attr.set_value(attr_CurrentSetPoint_read,1,0,false);

	if (attr_set_current)
		attr_set_current->set_write_value(attr_CurrentSetPoint_read);
}


//+----------------------------------------------------------------------------
//
// method : 		A2505BS::write_Current
// 
// description : 	Write Current attribute values to hardware.
//
//-----------------------------------------------------------------------------
void A2505BS::write_Current(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2505BS::write_Current(Tango::WAttribute &attr) entering... "<< endl;

	char tmp[STRING_SIZE];
	//	Add your own code to control device here
	attr.get_write_value(attr_Current_write);
	if (get_state() != Tango::ON)
	{
		Tango::Except::throw_exception (
				(const char *)"PS NOT ON",
		(const char *)"Cannot apply if PS not in ON state",
		(const char *)"A2505BS::write_Current()");
	}
	INFO_STREAM << "A2505BS::write_Current(): input = " << attr_Current_write << endl;
	if ((attr_Current_write < minCurr) || (attr_Current_write > maxCurr) )
	{
		Tango::Except::throw_exception (
			(const char *)"PARAMETER OUT OF RANGE",
			(const char *)"current value out of range",
			(const char *)"A2505BS::write_Current()");
	}
	sprintf (tmp, "MWI:%.3f", (attr_Current_write*1000.));
	dprintf("write_Current cmd: %s\n", tmp);
	
	try {
		sendCmd(tmp);
		getResponse();
		analyzeResponse();
	}
	catch (Tango::DevFailed &e) {
		TangoSys_MemStream out_stream;
		out_stream << "A2505BS::write current command failed" << ends;
		ERROR_STREAM <<out_stream.str()<<endl;
		Tango::Except::re_throw_exception(e,
		(const char *) "A2505BSDeviceException",
		out_stream.str(),
		(const char *) "A2505BSDeviceServer error", Tango::ERR);
	}
}


//+----------------------------------------------------------------------------
//
// method : 		A2505BS::read_PSStatus
// 
// description : 	Extract real attribute values for PSStatus acquisition result.
//
//-----------------------------------------------------------------------------
void A2505BS::read_PSStatus(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2505BS::read_PSStatus(Tango::Attribute &attr) entering... "<< endl;
	attr.set_value(attr_PSStatus_read, 8);
}

//+----------------------------------------------------------------------------
//
// method : 		A2505BS::read_Voltage
// 
// description : 	Extract real attribute values for Voltage acquisition result.
//
//-----------------------------------------------------------------------------
void A2505BS::read_Voltage(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2505BS::read_Voltage(Tango::Attribute &attr) entering... "<< endl;
	Tango::DevDouble argout = 0;
	try {
		sendCmd("MRV");
		getResponse();
		analyzeResponse(argout);
	}
	catch(Tango::DevFailed &e) {
		Tango::Except::re_throw_exception(e,
			(const char *)"Command failed",
			(const char *)"Error from A2505BS",
			(const char *)"A2505BS::read_Voltage()", Tango::ERR);
	}
	*attr_Voltage_read = (double)argout/1000;
	attr.set_value(attr_Voltage_read,1,0,false);
}

//+----------------------------------------------------------------------------
//
// method : 		A2505BS::read_Temperature
// 
// description : 	Extract real attribute values for Temperature acquisition result.
//
//-----------------------------------------------------------------------------
void A2505BS::read_Temperature(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2505BS::read_Temperature(Tango::Attribute &attr) entering... "<< endl;
	Tango::DevDouble argout = 0;
	try {
		sendCmd("MRT");
		getResponse();
		analyzeResponse(argout);
	}
	catch(Tango::DevFailed &e) {
		Tango::Except::re_throw_exception(e,
			(const char *)"Command failed",
			(const char *)"Error from A2505BS",
			(const char *)"A2505BS::read_Temperature()", Tango::ERR);
	}
	*attr_Temperature_read = (double)argout;
	attr.set_value(attr_Temperature_read,1,0,false);
}

//+----------------------------------------------------------------------------
//
// method : 		A2505BS::read_Current
// 
// description : 	Extract real attribute values for Current acquisition result.
//
//-----------------------------------------------------------------------------
void A2505BS::read_Current(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2505BS::read_Current(Tango::Attribute &attr) entering... "<< endl;
	Tango::DevDouble argout = 0;
	try {
		sendCmd("MRI");
		getResponse();
		analyzeResponse(argout);
	}
	catch(Tango::DevFailed &e) {
		Tango::Except::re_throw_exception(e,
			(const char *)"Command failed",
			(const char *)"Error from A2505BS",
			(const char *)"A2505BS::read_Current()", Tango::ERR);
	}
	*attr_Current_read = (double)argout/1000;
	attr.set_value(attr_Current_read,1,0,false);
	DEBUG_STREAM << "A2505BS::read_Current read also set point "<< endl;
	Tango::DevDouble argout2 = 0;
	try {
		sendCmd("MRG:36");
		getResponse();
		analyzeMRGResponse(argout2);
	}
	catch(Tango::DevFailed &e) {
		Tango::Except::re_throw_exception(e,
				(const char *)"Command failed",
				(const char *)"Error from A2505BS",
				(const char *)"A2505BS::read_Current()", Tango::ERR);
	}
	*attr_CurrentSetPoint_read = (double)argout2/1000;

	if (attr_set_current)
		attr_set_current->set_write_value(attr_CurrentSetPoint_read);
}


//+------------------------------------------------------------------
/**
 *	method:	A2505BS::get_id
 *
 *	description:	method to execute "GetID"
 *	Read identification (see Tabella 2
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevString A2505BS::get_id()
{
	//	POGO has generated a method core with argout allocation.
	//	If you would like to use a static reference without copying,
	//	See "TANGO Device Server Programmer's Manual"
	//		(chapter : Writing a TANGO DS / Exchanging data)
	//------------------------------------------------------------
	Tango::DevString	argout  = new char[200];
	DEBUG_STREAM << "A2505BS::get_id(): entering... !" << endl;

	//	Add your own code to control device here
	memset(argout,0,200);
	sendCmd ("MRID");
	getResponse ();
	strncpy (argout, response, 200);
	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	A2505BS::on
 *
 *	description:	method to execute "On"
 *	Power ON the power supply
 *	Returns reply in the boolean form: false for #NAK, true for #AK
 *
 *
 */
//+------------------------------------------------------------------
void A2505BS::on()
{
	DEBUG_STREAM << "A2505BS::on(): entering... !" << endl;
	//	Add your own code to control device here
	INFO_STREAM << "A2505BS::on()" << endl;
	int state = get_state();

	if ((state == Tango::OFF) || (state == Tango::STANDBY))
	{
		try {
			if (state == Tango::OFF)
				standby();
			dprintf ("ON: enabling PWM\n");
			sendCmd("MEN");
			getResponse();
			analyzeResponse();
		}
		catch (Tango::DevFailed &e) {
			TangoSys_MemStream out_stream;
			out_stream << "A2505BS: ON command failed" << ends;
			Tango::Except::re_throw_exception(e,
				(const char *) "A2505BSDeviceException",
				out_stream.str(),
				(const char *) "A2505BSDeviceServer error", Tango::ERR);
			ERROR_STREAM<<out_stream.str()<<endl;
		}
	}
	else
	{
		dprintf ("STATE ERROR: command on()\n");
		Tango::Except::throw_exception (
		(const char *)"STATE ERROR",
		(const char *)"Already in ON state",
		(const char *)"Already in ON state");
	}
}

//+------------------------------------------------------------------
/**
 *	method:	A2505BS::off
 *
 *	description:	method to execute "Off"
 *	Power OFF the power supply.
 *
 *
 */
//+------------------------------------------------------------------
void A2505BS::off()
{
	DEBUG_STREAM << "A2505BS::off(): entering... !" << endl;
	//	Add your own code to control device here
	INFO_STREAM << "A2505BS::off()" << endl;
	dprintf ("OFF: disabling PWM\n");
	try {
		sendCmd ("MDIS");
		//dprintf("** get response\n");
		getResponse();
		//dprintf("** analyze response\n");
		analyzeResponse();
		//dprintf("** end of all\n");
	}
	catch (Tango::DevFailed &e) {
		TangoSys_MemStream out_stream;
		out_stream << "A2505BS: disabling command failed" << ends;
		Tango::Except::re_throw_exception(e,
				(const char *) "A2505BSDeviceException",
				out_stream.str(),
				(const char *) "A2505BSDeviceServer error", Tango::ERR);
		ERROR_STREAM<<out_stream.str()<<endl;
	}
	dprintf ("OFF: opening rele\n");
	try {
		sendCmd ("MOFF");
		getResponse();
		analyzeResponse();
	}
	catch (Tango::DevFailed &e) {
		TangoSys_MemStream out_stream;
		out_stream << "A2505BS: opening rele command failed" << ends;
		Tango::Except::re_throw_exception(e,
				(const char *) "A2505BSDeviceException",
				out_stream.str(),
				(const char *) "A2505BSDeviceServer error", Tango::ERR);
		ERROR_STREAM<<out_stream.str()<<endl;
	}
}

//+------------------------------------------------------------------
/**
 *	method:	A2505BS::mod_glob_stat
 *
 *	description:	method to execute "ModGlobStat"
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevString A2505BS::mod_glob_stat()
{
	//	POGO has generated a method core with argout allocation.
	//	If you would like to use a static reference without copying,
	//	See "TANGO Device Server Programmer's Manual"
	//		(chapter : Writing a TANGO DS / Exchanging data)
	//------------------------------------------------------------
	Tango::DevString	argout  = new char[200];
	DEBUG_STREAM << "A2505BS::mod_glob_stat(): entering... !" << endl;

	//	Add your own code to control device here
	memset(argout, 0, 200);
	sendCmd ("MGLST");
	getResponse();
	strncpy (argout, response, 200);
	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	A2505BS::disable_switcher
 *
 *	description:	method to execute "DisableSwitcher"
 *	Returns reply in the boolean form: false for #NAK, true for #AK
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevBoolean A2505BS::disable_switcher()
{
	Tango::DevBoolean	argout ;
	DEBUG_STREAM << "A2505BS::disable_switcher(): entering... !" << endl;

	//	Add your own code to control device here
	INFO_STREAM << "A2505BS::disable_switcher()" << endl;
	sendCmd ("MDIS");
	getResponse();
	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	A2505BS::enable_switcher
 *
 *	description:	method to execute "EnableSwitcher"
 *	Returns reply in the boolean form: false for #NAK, true for #AK
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevBoolean A2505BS::enable_switcher()
{
	Tango::DevBoolean	argout ;
	DEBUG_STREAM << "A2505BS::enable_switcher(): entering... !" << endl;

	//	Add your own code to control device here
	INFO_STREAM << "A2505BS::enable_switcher()" << endl;
	sendCmd ("MEN");
	getResponse();
	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	A2505BS::clear_status
 *
 *	description:	method to execute "ClearStatus"
 *	Returns reply in the boolean form: false for #NAK, true for #AK
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevBoolean A2505BS::clear_status()
{
	Tango::DevBoolean	argout ;
	DEBUG_STREAM << "A2505BS::clear_status(): entering... !" << endl;

	//	Add your own code to control device here
	INFO_STREAM << "A2505BS::clear_status()" << endl;
	sendCmd ("MCLR");
	getResponse();
	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	A2505BS::standby
 *
 *	description:	method to execute "Standby"
 *	Goto into STANDBY mode.
 *
 *
 */
//+------------------------------------------------------------------
void A2505BS::standby()
{
	DEBUG_STREAM << "A2505BS::standby(): entering... !" << endl;
		
	//	Add your own code to control device here
	INFO_STREAM << "A2505BS::standby()" << endl;
	char tmp[STRING_SIZE];
	memset(tmp,0,STRING_SIZE);
	if (get_state() == Tango::OFF)
	{
		try {
			dprintf ("MRESET in standby()\n");
			reset();
			dprintf ("MWI in standby()\n");
			sprintf (tmp, "MWI:%.3f", standbyCurrent);
			dprintf ("standbyCurrent: %.3f\n", standbyCurrent);
			sendCmd(tmp);
			getResponse();
			analyzeResponse();
			dprintf ("MON in standby()\n");
			sendCmd("MON");
			getResponse();
			analyzeResponse();
		}
		catch(Tango::DevFailed &e) {
			Tango::Except::re_throw_exception(e,
					(const char *)"Command failed",
					(const char *)"Error from A2505BS",
					(const char *)"A2505BS::OFF->STANDBY", Tango::ERR);
					ERROR_STREAM<<"A2505BS::standby() command failed" <<endl;
		}
	}
	else if (get_state() == Tango::ON)
	{
		try {
			sprintf (tmp, "MWI:%.3f", standbyCurrent);
			dprintf ("standbyCurrent: %.3f\n", standbyCurrent);
			sendCmd(tmp);
			getResponse();
			analyzeResponse();
			dprintf ("MDIS in standby()\n");
			sendCmd("MDIS");
			getResponse();
			analyzeResponse();
		}
		catch(Tango::DevFailed &e) {
			Tango::Except::re_throw_exception(e,
					(const char *)"Command failed",
					(const char *)"Error from A2505BS",
					(const char *)"A2505BS::ON->STANDBY", Tango::ERR);
					ERROR_STREAM<<"A2505BS::standby() command failed" <<endl;
		}
		
	}
}

//+------------------------------------------------------------------
/**
 *	method:	A2505BS::get_current
 *
 *	description:	method to execute "GetCurrent"
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevDouble A2505BS::get_current()
{
	Tango::DevDouble	argout ;
	DEBUG_STREAM << "A2505BS::get_current(): entering... !" << endl;

	//	Add your own code to control device here
	sendCmd ("MRI");
	getResponse();
	analyzeResponse(argout);
	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	A2505BS::get_voltage
 *
 *	description:	method to execute "GetVoltage"
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevDouble A2505BS::get_voltage()
{
	Tango::DevDouble	argout ;
	DEBUG_STREAM << "A2505BS::get_voltage(): entering... !" << endl;

	//	Add your own code to control device here
	sendCmd ("MRV");
	getResponse();
	analyzeResponse(argout);
	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	A2505BS::get_dacvalue
 *
 *	description:	method to execute "GetDACValue"
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevDouble A2505BS::get_dacvalue()
{
	Tango::DevDouble	argout ;
	DEBUG_STREAM << "A2505BS::get_dacvalue(): entering... !" << endl;

	//	Add your own code to control device here
	sendCmd ("MRD");
	getResponse();
	analyzeResponse(argout);
	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	A2505BS::get_reference
 *
 *	description:	method to execute "GetReference"
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevDouble A2505BS::get_reference()
{
	Tango::DevDouble	argout ;
	DEBUG_STREAM << "A2505BS::get_reference(): entering... !" << endl;

	//	Add your own code to control device here
	sendCmd ("MRR");
	getResponse();
	analyzeResponse(argout);
	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	A2505BS::get_temperature
 *
 *	description:	method to execute "GetTemperature"
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevDouble A2505BS::get_temperature()
{
	Tango::DevDouble	argout ;
	DEBUG_STREAM << "A2505BS::get_temperature(): entering... !" << endl;

	//	Add your own code to control device here
	//exec("MRT", "A2505BS::get_temperature()");
	try {
		sendCmd("MRT");
		getResponse();
		analyzeResponse(argout);
	}
	catch(Tango::DevFailed &e) {
		Tango::Except::re_throw_exception(e,
				(const char *)"Command failed",
				(const char *)"Error from A2505BS",
				(const char *)"A2505BS::get_temperature()", Tango::ERR);
	}

	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	A2505BS::get_positive_rail
 *
 *	description:	method to execute "GetPositiveRail"
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevDouble A2505BS::get_positive_rail()
{
	Tango::DevDouble	argout ;
	DEBUG_STREAM << "A2505BS::get_positive_rail(): entering... !" << endl;

	//	Add your own code to control device here
	//exec("MRP", "A2505BS::get_positive_rail()");
	sendCmd("MRP");
	getResponse();
	analyzeResponse(argout);
	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	A2505BS::get_status
 *
 *	description:	method to execute "GetStatus"
 *
 *
 */
//+------------------------------------------------------------------
void A2505BS::get_status()
{
	DEBUG_STREAM << "A2505BS::get_status(): entering... !" << endl;

	//	Add your own code to control device here
	long status, bitmask = 1;
	int i;

	//dprintf("mst()\n");
	try
	{
		sendCmd("MST");
		getResponse();
		char *result = NULL;
		result = strtok (response, ":");
		result = strtok (NULL, ":");
		if (result != NULL)
			status = strtol (result, NULL, 16);
	}
	catch(Tango::DevFailed &e) {
		//set_state(Tango::FAULT);
		//set_status("Device FAULT, communication broken");
		dprintf("EXCEPTION CATCHED\n");
		ERROR_STREAM<<"A2505BS::Communication broken"<<endl;
		Tango::Except::re_throw_exception(e,
		(const char *)"Device FAULT",
		(const char *)"Communication broken",
		(const char *)"A2505BS::get_status()", Tango::ERR);
	}
	if (get_state() == Tango::ALARM)
		return;
	//dprintf("status: %ld\n", status);
	for (i = 0; i < 8; i++) {
		statusArray[i]=0;
		if (status & bitmask) {
			statusArray[i]=1;
#ifdef DEBUG
			switch (i) {
				case 0:
					dprintf("PS OFF\n");
					break;
				case 1:
					dprintf("PS ON\n");
					break;
				case 2:
					dprintf("PWM disabled\n");
					break;
				case 3:
					dprintf("PWM enabled\n");
					break;
				case 4:
					dprintf("PS locked or interlock\n");
					break;
				case 5:
					dprintf("RESET done\n");
					cout<<"RESET done"<<endl;
					break;
				case 6:
					dprintf("over temp\n");
					break;
				case 7:
					dprintf("DC-link fault\n");
					break;
				default:
					dprintf("error: 0 bit set\n");
					break;
			}
#endif
		}
		bitmask *= 2;
	}

	if (statusArray[4] || statusArray[6] || statusArray[7])
	{
		//fault
		set_state(Tango::FAULT);
		if (statusArray[4] && statusArray[7])
			set_status("DC-link FAULT");
		if (statusArray[4] && statusArray[7] && statusArray[6])
			set_status("DC-link & OVER TEMP FAULT");
		if (statusArray[4] && statusArray[6])
			set_status("OVER TEMP FAULT");
		if (statusArray[4])
			set_status("MAGNET FAULT");
	}
	else {
		//standby
		if (statusArray[1] && statusArray[2])
		{
			set_state(Tango::STANDBY);
			set_status("PS STANDBY (PWM DISABLED)");
		}
		//on
		if (statusArray[1] && statusArray[3])
		{
			set_state(Tango::ON);
			set_status("PS ON (PWM ENABLED)");
		}
		//reset
		if (statusArray[0] && statusArray[2] && statusArray[5])
		{
			set_state(Tango::OFF);
			set_status("RESET");
		}
		//off
		else if (statusArray[0])
		{
			set_state(Tango::OFF);
			set_status("PS OFF");
		}
	}
}

//+------------------------------------------------------------------
/**
 *	method:	A2505BS::reset
 *
 *	description:	method to execute "Reset"
 *
 *
 */
//+------------------------------------------------------------------
void A2505BS::reset()
{
	DEBUG_STREAM << "A2505BS::reset(): entering... !" << endl;

	//	Add your own code to control device here
	INFO_STREAM << "A2505BS::reset()" << endl;
	try {
		sendCmd("MRESET");
		getResponse();
		getResponse();
		analyzeResponse();
	}
	catch (Tango::DevFailed &e) {
		TangoSys_MemStream out_stream;
		out_stream << "Reset command failed" << ends;
		Tango::Except::re_throw_exception(e,
		(const char *)"Command failed",
		(const char *)"Error from A2505BS",
		(const char *)"A2505BS::reset()", Tango::ERR);
		ERROR_STREAM<<"A2505BS::reset() command failed"<<endl;
	}
}

void A2505BS::fileLogMsg(char *msg)
{
	time_t ltime;
	struct tm *Tm;
	struct timeval detail;
	char tmp[STRING_SIZE];
	
	ltime=time(NULL);
	Tm=localtime(&ltime);
	gettimeofday(&detail,NULL);

	sprintf(tmp, "%d-%.2d-%.2d %.2d:%.2d:%.2d.%.3ld: %s %s",
		Tm->tm_year+1900,
		Tm->tm_mon+1,
		Tm->tm_mday,
		Tm->tm_hour,
		Tm->tm_min,
		Tm->tm_sec,
		detail.tv_usec/1000,
		get_name().c_str(),
		msg);
	cout<<tmp<<endl;
}

//+------------------------------------------------------------------
/**
 *	method:	A2505BS::start_cycling
 *
 *	description:	method to execute "StartCycling"
 *	Begins cycling procedure: [to be described once knew]
 *
 *
 */
//+------------------------------------------------------------------
void A2505BS::start_cycling()
{
	DEBUG_STREAM << "A2505BS::start_cycling(): entering... !" << endl;

	//	Add your own code to control device here
	if (get_state() != Tango::ON)
	{
		Tango::Except::throw_exception (
				(const char *)"PS NOT ON",
		(const char *)"Cannot apply if PS not in ON state",
		(const char *)"A2505BS::start_cycling()");
	}
	INFO_STREAM <<"Cycling started, m/M/e: "
	<<cycMinCurr<<" "<<cycMaxCurr<<" "<<cycEndCurr<<endl;
	
	writeCycleCurrent(cycMinCurr);
	usleep(cycleWait*1000);

	writeCycleCurrent(cycMaxCurr);
	usleep(cycleWait*1000);

	writeCycleCurrent(cycEndCurr);
	usleep(cycleWait*1000);
	INFO_STREAM <<"System9000:Cycling finished"<<endl;
}

//+------------------------------------------------------------------
/**
 *	method:	A2505BS::abort
 *
 *	description:	method to execute "Abort"
 *	Abort cycling.
 *
 *
 */
//+------------------------------------------------------------------
void A2505BS::abort()
{
	DEBUG_STREAM << "A2505BS::abort(): entering... !" << endl;

	//	Add your own code to control device here
	dprintf("Abort(): not implemented yet.\n");
}

void A2505BS::writeCycleCurrent(Tango::DevDouble cur)
{
	DEBUG_STREAM << "A2505BS::writeCycleCurrent: entering... ! "<< endl;

	char tmp[STRING_SIZE];
	INFO_STREAM << "A2505BS::writeCycleCurrent input = " << cur << endl;
	if ((cur < minCurr) || (cur > maxCurr) )
	{
		Tango::Except::throw_exception (
			(const char *)"PARAMETER OUT OF RANGE",
			(const char *)"current value out of range",
			(const char *)"A2505BS::writeCycleCurrent()");
	}
	sprintf (tmp, "MWI:%.3f", (cur*1000.));
	dprintf("writeCycleCurrent cmd: %s\n", tmp);
	
	try {
		sendCmd(tmp);
		getResponse();
		analyzeResponse();
	}
	catch (Tango::DevFailed &e) {
		TangoSys_MemStream out_stream;
		out_stream << "A2505BS::writeCycleCurrent command failed" << ends;
		Tango::Except::re_throw_exception(e,
		(const char *) "A2505BSDeviceException",
		out_stream.str(),
		(const char *) "A2505BSDeviceServer error", Tango::ERR);
		ERROR_STREAM <<out_stream.str()<<endl;
	}
}

}	//	namespace
