//=============================================================================
//
// file :        A2505BS.h
//
// description : Include for the A2505BS class.
//
// project :	Preinjector corrector PS
//
// $Author: claudio $
//
// $Revision: 1.3 $
//
// $Log: A2505BS.h,v $
// Revision 1.3  2019-11-13 12:53:43  claudio
// TANGO_BASE_CLASS, tango-9.3.3
//
// Revision 1.2  2013-09-03 07:57:12  giulio
// Moved to Tango8 and Device4Impl
//
// Revision 1.1  2009-10-13 08:55:33  graziano
// first commit
//
// Revision 1.1  2007/12/12 11:44:38  enrico
// Release version. ME
//
//
// copyleft :   Sincrotrone Trieste S.C.p.A.
//              Strada Statale 14 - km 163,5 in AREA Science Park
//              34012 Basovizza, Trieste
//              ITALY
//
//=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================
#ifndef _A2505BS_H
#define _A2505BS_H

#include <tango.h>


#include <sys/time.h>
#include <time.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

// 0xA
#define LF 10
// 0xD
#define CR 13

#define STRING_SIZE 256
#define CYCLE_WAIT  400000   //400 ms


const short BUF_LEN=512;

/**
 * @author	$Author: claudio $
 * @version	$Revision: 1.3 $
 */

 //	Add your own constants definitions here.
 //-----------------------------------------------


namespace A2505BS_ns
{

/**
 * Class Description:
 * Tango device for the preinjector corrector power supply system (model A2505BS
 */

/*
 *	Device States Description:
 *  Tango::ON :       Power ON
 *  Tango::OFF :      Power OFF
 *  Tango::STANDBY :  STANDBY
 *  Tango::FAULT :    *
 *  Tango::ALARM :    When no socket connection (timeout on recv)
 */


class A2505BS: public TANGO_BASE_CLASS
{
public :
	//	Add your own data members here
	//-----------------------------------------


	//	Here is the Start of the automatic code generation part
	//-------------------------------------------------------------	
/**
 *	@name attributes
 *	Attributs member data.
 */
//@{
		Tango::DevDouble	*attr_Voltage_read;
		Tango::DevDouble	*attr_Temperature_read;
		Tango::DevDouble	*attr_Current_read;
		Tango::DevDouble	attr_Current_write;
		Tango::DevBoolean	*attr_PSStatus_read;
		Tango::DevDouble	*attr_CurrentSetPoint_read;
		Tango::DevLong	*attr_ModelID_read;
//@}

/**
 *	@name Device properties
 *	Device properties member data.
 */
//@{
/**
 *	IP address of power supply control device
 */
	string	ipAddress;
/**
 *	TCP port serving the power supply control device
 */
	Tango::DevShort	tcpPort;
/**
 *	Current imposed on standby command.
 */
	Tango::DevDouble	standbyCurrent;
/**
 *	You guess. Max current value.
 */
	Tango::DevDouble	maxCurr;
/**
 *	You guess. Min current value.
 */
	Tango::DevDouble	minCurr;
/**
 *	
 */
	Tango::DevDouble	cycMaxCurr;
/**
 *	
 */
	Tango::DevDouble	cycMinCurr;
/**
 *	
 */
	Tango::DevDouble	cycEndCurr;
/**
 *	milliseconds
 */
	Tango::DevShort	cycleWait;
//@}

/**@name Constructors
 * Miscellaneous constructors */
//@{
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device Name
 */
	A2505BS(Tango::DeviceClass *cl,string &s);
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device Name
 */
	A2505BS(Tango::DeviceClass *cl,const char *s);
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device name
 *	@param d	Device description.
 */
	A2505BS(Tango::DeviceClass *cl,const char *s,const char *d);
//@}

/**@name Destructor
 * Only one desctructor is defined for this class */
//@{
/**
 * The object desctructor.
 */	
	~A2505BS() {delete_device();};
/**
 *	will be called at device destruction or at init command.
 */
	void delete_device();
//@}

	
/**@name Miscellaneous methods */
//@{
/**
 *	Initialize the device
 */
	virtual void init_device();
/**
 *	Always executed method befor execution command method.
 */
	virtual void always_executed_hook();

//@}

/**
 * @name A2505BS methods prototypes
 */

//@{
/**
 *	Hardware acquisition for attributes.
 */
	virtual void read_attr_hardware(vector<long> &attr_list);
/**
 *	Extract real attribute values for Voltage acquisition result.
 */
	virtual void read_Voltage(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Temperature acquisition result.
 */
	virtual void read_Temperature(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Current acquisition result.
 */
	virtual void read_Current(Tango::Attribute &attr);
/**
 *	Write Current attribute values to hardware.
 */
	virtual void write_Current(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for PSStatus acquisition result.
 */
	virtual void read_PSStatus(Tango::Attribute &attr);
/**
 *	Extract real attribute values for CurrentSetPoint acquisition result.
 */
	virtual void read_CurrentSetPoint(Tango::Attribute &attr);
/**
 *	Extract real attribute values for ModelID acquisition result.
 */
	virtual void read_ModelID(Tango::Attribute &attr);
/**
 *	Read/Write allowed for Voltage attribute.
 */
	virtual bool is_Voltage_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Temperature attribute.
 */
	virtual bool is_Temperature_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Current attribute.
 */
	virtual bool is_Current_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for PSStatus attribute.
 */
	virtual bool is_PSStatus_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for CurrentSetPoint attribute.
 */
	virtual bool is_CurrentSetPoint_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for ModelID attribute.
 */
	virtual bool is_ModelID_allowed(Tango::AttReqType type);
/**
 *	Execution allowed for GetID command.
 */
	virtual bool is_GetID_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for On command.
 */
	virtual bool is_On_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Off command.
 */
	virtual bool is_Off_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for GetCurrent command.
 */
	virtual bool is_GetCurrent_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for GetVoltage command.
 */
	virtual bool is_GetVoltage_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for GetDACValue command.
 */
	virtual bool is_GetDACValue_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for GetReference command.
 */
	virtual bool is_GetReference_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for GetTemperature command.
 */
	virtual bool is_GetTemperature_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for GetPositiveRail command.
 */
	virtual bool is_GetPositiveRail_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for GetStatus command.
 */
	virtual bool is_GetStatus_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Reset command.
 */
	virtual bool is_Reset_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for ModGlobStat command.
 */
	virtual bool is_ModGlobStat_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for DisableSwitcher command.
 */
	virtual bool is_DisableSwitcher_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for EnableSwitcher command.
 */
	virtual bool is_EnableSwitcher_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for ClearStatus command.
 */
	virtual bool is_ClearStatus_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Standby command.
 */
	virtual bool is_Standby_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for StartCycling command.
 */
	virtual bool is_StartCycling_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Abort command.
 */
	virtual bool is_Abort_allowed(const CORBA::Any &any);
/**
 * Read identification (see Tabella 2
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevString	get_id();
/**
 * Power ON the power supply
 *	Returns reply in the boolean form: false for #NAK, true for #AK
 *	@exception DevFailed
 */
	void	on();
/**
 * Power OFF the power supply.
 *	@exception DevFailed
 */
	void	off();
/**
 * 
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevDouble	get_current();
/**
 * 
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevDouble	get_voltage();
/**
 * 
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevDouble	get_dacvalue();
/**
 * 
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevDouble	get_reference();
/**
 * 
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevDouble	get_temperature();
/**
 * 
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevDouble	get_positive_rail();
/**
 * 
 *	@exception DevFailed
 */
	void	get_status();
/**
 * 
 *	@exception DevFailed
 */
	void	reset();
/**
 * 
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevString	mod_glob_stat();
/**
 * Returns reply in the boolean form: false for #NAK, true for #AK
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevBoolean	disable_switcher();
/**
 * Returns reply in the boolean form: false for #NAK, true for #AK
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevBoolean	enable_switcher();
/**
 * Returns reply in the boolean form: false for #NAK, true for #AK
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevBoolean	clear_status();
/**
 * Goto into STANDBY mode.
 *	@exception DevFailed
 */
	void	standby();
/**
 * Begins cycling procedure: [to be described once knew]
 *	@exception DevFailed
 */
	void	start_cycling();
/**
 * Abort cycling.
 *	@exception DevFailed
 */
	void	abort();

/**
 *	Read the device properties from database
 */
	 void get_device_property();
//@}

	//	Here is the end of the automatic code generation part
	//-------------------------------------------------------------	



protected :	
	//	Add your own data members here
	//-----------------------------------------
	void		init_attr_double(Tango::DevDouble *&);
	void		init_attr_long(Tango::DevLong *&);
	void		openSocket();
	void		sendCmd(Tango::DevString);
	void		getResponse();
	void		analyzeResponse(Tango::DevDouble&);
	void		analyzeResponse(Tango::DevLong&);
	void		analyzeResponse();
	void		analyzeMRGResponse(Tango::DevDouble&);
	void		writeCycleCurrent(Tango::DevDouble);
	void		fileLogMsg(char *);
	char		recvBuf[BUF_LEN];
	char		response[BUF_LEN];
	int		sockfd;
	bool		first_time;
	struct		timeval	tv;
	Tango::DevBoolean	statusArray[8];
	struct sockaddr_in	serv_addr;
	Tango::WAttribute	*attr_set_current;
};

}	// namespace_ns

#endif	// _A2505BS_H
